<?php
/************************************************
 * 申し込み受付・集計用のPHPプログラム
 * バージョン v0.3 (2006-10-27)，趙，京都大学
 * ライセンス：GPLv2
 * 注：名前をindex.phpにすること．要 PHP + GD
 ************************************************/

//
// 定数定義（名前が大文字）
//

// イベントの総称．文字列一つ，HTMLタグなしで．
$EVENT_TITLE = "AL・CAS・CST研究会 懇親会・昼食会";

// イベントに関する前書き，文字列一つ，インラインタグOK
$EVENT_INTRODUCTION = "研究会ホームページ [<a href=\"http://www.ipsj.or.jp/sig/al/\">AL</a>] [<a href=\"http://www.ieice.org/ess/cas/\">CAS</a>] [<a href=\"http://www.ieice.org/~cst/\">CST</a>] プログラム [<a href=\"http://www.ipsj.or.jp/sig/al/prog18/prog109.html\">AL</a>] [<a href=\"http://www.ieice.org/ken/program/index.php?tgs_regid=a08d3a99eed7a66fba9394bad784fc773b504a8949686c47f8090082ef76e030&tgid=CAS&lang=\">CAS/CST</a>]";

// イベントの名称．複数可（以下の配列も同じサイズで）．
$EVENT_NAME = array("懇親会", "昼食会");

// イベントの日程．
$EVENT_TIME = array("11月20日（月）18:30 - 20:30", "11月21日（火）12:00 - 13:30");

// イベントの場所．HTMLインライン要素（リンクなど）可
$EVENT_PLACE = array("<a href=\"http://www.tebaya.jp/frame.htm\">手羽屋長崎総本店</a>", "学生食堂２階ホール");

// イベントの費用（予算）
$EVENT_COST = array("５０００円程度", "会費無料");

// イベントについての備考，HTMLインライン要素可．
$EVENT_MISC = array("<strong>平目の活き作りがつきます</strong>", "");

// イベントに関する後書き（連絡先とか），文字列一つ，インラインタグOK
$EVENT_FOOTER = "懇親会・昼食会についての連絡先：○○○";

// 申し込みに関する問い合わせ先．
// トラブルがあったときに送信されるので，正確なEMAILアドレスを入れること．
$ADMIN_NAME = "京都大学情報学研究科 趙";
$ADMIN_EMAIL = "liang@i.kyoto-u.ac.jp";

// 申し込みの〆切，一つで．mktimeの引数：時，分，秒，月，日，年
$DEADLINE = mktime(23, 59, 59, 11, 12, 2015);

// キャンセルした人の記録を見せるか？ true = 見せる, false = 見せない．
$SHOW_CANCEL = false;

// Webサーバ用の文字コードに合わせて下さい．utf-8を強く勧める．
$ENCODE = "utf-8";

// データファイル（ウェブデーモンが読め書き可能でなくてはならない）
// セキュリティに関する注：ネットから直接見れないところにおくこと．
$DATA_FILE = "./data/phpusher.csv";

////////////////////////////////////////////////////////////////////
// 警告：                                                         //
// プログラムをよく分かっている人でなければ，以下を変えないこと！ //
////////////////////////////////////////////////////////////////////

// プロジェクトバージョン情報
$PROJECT = "phpUsher 0.3 (2006-10-27), L. Zhao &lt;liang \at i.kyoto-u.ac.jp&gt;";

// イベントの数
$EVENT_NUM = count($EVENT_NAME);

// データファイルの形式：コンマ区切りCSV，一行最大1024バイト
//   氏名,所属,E-mail,0/1,0/1,...,申込日時,確認コード,状態
// ただし，氏名，所属，E-mailは入力通り．
// 0/1の数は$EVENT_NUMと同じ，イベントに参加する場合 1 ，しない場合 0
// 申込日時は，申込時点のUNIX時刻
// 確認コード：氏名所属E-mail申込時刻のmd5ハッシュ値（32文字）
// 状態：負数＝キャンセル時刻（未実装），0＝確認待ち，正数＝確認時刻

//
// グローバル変数（定数ではない）
//

// データをソートする際に使うフィールド（0から．無効値はソートしない）．
// 関数 data_compare() 参照．
$SORT_FIELD = 0;

//
// 関数定義
//

// function: get_baseURI
// @return the base URI of this server (Shouldn't PHP set it?!)
// @todo: 2005-04-20, no better way?
function get_baseURI() {
	$server = $_SERVER['SERVER_NAME'];
	$port = $_SERVER['SERVER_PORT'];
	if (isset($_SERVER['HTTPS'])) { // https, default port is 443
		$proto = "https://";
		$port = ($port == 443) ? "" : ":$port";
	}
	else { // http, default port is 80
		$proto = "http://";
		$port = ($port == 80) ? "" : ":$port";
	}
	return "$proto$server$port";
}

// function: print_header
function print_header() {
	global $ENCODE, $EVENT_TITLE, $PROJECT;

	header("Content-Type: text/html; charset=$ENCODE");
	print<<<EOF
<?xml version="1.0" encoding="$ENCODE"?>
<!DOCTYPE html
    PUBLIC "-//W3C//DTD XHTML 1.1 //EN"
    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=$ENCODE" />
    <title>$EVENT_TITLE</title>
    <link rel="stylesheet" href="style.css" type="text/css" />
  </head>
  <body>
    <h1>$EVENT_TITLE</h1>
EOF;
    // <h1><img src="logo.png" width="206" height="78" alt="phpUsher" /></h1>
}

// function: print_footer
// @param $cmd nonempty to show the address line (empty for security)
function print_footer($cmd="") {
	global $PROJECT, $ADMIN_NAME, $ADMIN_EMAIL;

	$admin = $ADMIN_NAME." &lt;".str_replace("@", " \\at ", $ADMIN_EMAIL)."&gt;";

	if ($cmd != "") {
		print<<<EOF
    <p>申し込みシステムに関するトラブルや御意見などは，$admin まで御連絡下さい．</p>
    <address>$PROJECT</address>
  </body>
</html>
EOF;
	}
	else {
		print("<hr /></body></html>");
	}
}

// function: quit
// @param $err error message to show (must be a valid child elemet of <body>).
function quit($err="<p>すみません，内部エラーが発生しました．</p>") {
	print_header();
	print($err);
	print_footer("show address");
	exit();
}

// function: parse_post
// @return array (name, affiliation, mail, event_check, ...). NULL for invalid post.
// @todo 2006-08-23 Shall we check the validness of the mail address?
function parse_post() {
	global $EVENT_NUM;

	if (!(isset($_POST['name']) && isset($_POST['affiliation']) && isset($_POST['mail']) && isset($_POST['spamcode']))) {
		return NULL;
	}
	$val = array();
	array_push($val, trim($_POST['name']));
	array_push($val, trim($_POST['affiliation']));
	array_push($val, trim($_POST['mail']));
	$type = array();
	for ($i=0; $i<$EVENT_NUM; $i++) {
		$type[$i] = 0;
	}
	while(list($i, $value) = each($_POST)) {
		if (strcasecmp($value, "type") == 0 && $i>=0 && $i<$EVENT_NUM) {
			$type[$i] = 1;
		}
	}
	array_push($val, $type);
	array_push($val, trim($_POST['spamcode']));
	return $val;
}

// funtion: data_compare
// Given two data, compare them according to $SORT_FIELD
function data_compare($data1, $data2) {
	global $SORT_FIELD;

	$n = count($data1);
	if ($n != count($data2) || $n <= $SORT_FIELD || $SORT_FIELD < 0) return 0;
	$val = strcasecmp($data1[$SORT_FIELD], $data2[$SORT_FIELD]);
	return ($val>0) ? 1 : (($val<0)?(-1):0);
}

//
// main
//

// get our URI. We hide the script name to increase security (only) a bit.
$myURI = get_baseURI().dirname($_SERVER['PHP_SELF']);

// We must set the locale, otherwise fgetcsv may not work.
setlocale(LC_ALL, "ja_JP.".strtoupper($ENCODE));

// self check: quit if the deadline has expired.
if ($DEADLINE <= time()) {
	quit("<p>〆切を過ぎましたので，受付は終了しました．</p>");
}

// self check: create the data file if it does not exist.
// Then test if the file exists. If not, we have an error.
if (!file_exists($DATA_FILE)) {
	$file = @fopen($DATA_FILE, "w+");
	@fclose($file);
}
if (!(is_readable($DATA_FILE) && is_writable($DATA_FILE))) {
	quit("<h2>インストールエラー</h2><p>データファイル（DATA_FILE）の読み書きにエラーが発生しました．</p>");
}

// OK, self check is fine.
session_start();
// Check if we have query.
@parse_str($_SERVER['QUERY_STRING']);

// output the anti-spam image
if (isset($image)) {
	if (!isset($_SESSION['passcode'])) {
		$_SESSION['passcode'] = mt_rand(1000, 9999);
	}
	$img = imagecreatetruecolor(60, 30);
	$background = imagecolorallocate($img, 0, 0, 255);
	$foreground = imagecolorallocate($img, 255, 255, 255);
	imagefilledrectangle($img, 2, 2, 58, 28, $background);
	imagestring($img, 5, 10, 5, $_SESSION['passcode'], $foreground);
	imagecolordeallocate($img, $background);
	imagecolordeallocate($img, $foreground);
	header("Content-Type: image/png");
	imagepng($img);
	imagedestroy($img);
	exit();
}

print_header();
print("<h2>御案内</h2>");
if ($EVENT_INTRODUCTION != "") {
	print("<p>".$EVENT_INTRODUCTION."</p>");
}
print("<div class=\"box\"><table>");
$targets = array("名称"=>"EVENT_NAME", "日程"=>"EVENT_TIME", "場所"=>"EVENT_PLACE", "参加費"=>"EVENT_COST", "備考"=>"EVENT_MISC");
foreach($targets as $key=>$val) {
	print("<tr><td>$key: </td>");
	$items = $$val;
	for($i=0; $i<$EVENT_NUM; $i++) {
		print("<td>".$items[$i]."</td>");
	}
	print("</tr>");
}
print("</table></div>");
if ($EVENT_FOOTER != "") {
	print("<p>".$EVENT_FOOTER."</p>");
}

if (!isset($action)) { // No action
	$day = getdate($DEADLINE);
	$DAYS = array("日", "月", "火", "水", "木", "金", "土");
	print("<h2>参加申込</h2><p>申込〆切： <strong>".$day['year']." 年 ".$day['mon']." 月 ".$day['mday']." 日（".$DAYS[$day['wday']]."）</strong> &gt;&gt;&gt; <a href=\"?action=list\">参加者一覧</a></p>");
	print<<<EOF
	<p>ステップ１．氏名，所属とE-mailアドレスを入力します →  参加するイベントをチェックします →  SPAM対策コードを入力します → 「申込」ボタンを押します．</p>
	<form method="post" action="?action=guide">
	<div class="color">
	<table>
		<tr><td>氏名: </td><td><input type="text" name="name" size="60" /></td></tr>
		<tr><td>所属: </td><td><input type="text" name="affiliation" size="60" /></td></tr>
		<tr><td>E-mail（半角）: </td><td><input type="text" name="mail" size="60" /></td></tr>
		<tr><td>参加: </td><td>
EOF;
	for($i=0; $i<$EVENT_NUM; $i++) {
		print("<label><input type=\"checkbox\" name=\"$i\" value=\"type\" checked=\"checked\" />".$EVENT_NAME[$i]."</label> ");
	}
	$n = mt_rand(0, 100000000); // dummy to cheat proxy/browser's cache manager.
	print<<<EOF
		<tr><td><img src="?image=$n" alt="" /><td><label>左のSPAM対策コード（半角数字４桁） <input type="text" name="spamcode" size="24" /></label></td></tr>
	</td></tr>
	</table>
	<input type="submit" value="申込" />
	</div>
	</form>
EOF;
}
else { // Otherwise we have action query.
	switch ($action) {
		///////////////////////////////////////////////////////////////////
		// confirmation
		case "guide":
			$post = parse_post();
			if ($post != NULL) {
				$spam = array_pop($post);
				if (!isset($_SESSION['passcode']) || strcmp($_SESSION['passcode'], $spam) != 0) {
					sleep(1); // pause a while to protect us from spammers
					print("<h2>申込エラー</h2><p>無効なSPAM対策コードです．</p>");
					break;
				};
				$type = array_pop($post);
				$mail = array_pop($post);
				if (!eregi('[a-z0-9@.-_+]+', $mail)) { // not valid email address
					print("<h2>申込エラー</h2><p>無効なEmailアドレスです（半角の英数字@.-_+の組合せになっていますか？）．</p>");
					break;
				};
				$affi = array_pop($post);
				$name = array_pop($post);
				if ($name != "" && $mail != "") {
					unset($_SESSION['passcode']);
					$date = time();
					$code = md5("$name$affi$mail$date");
					$fp = fopen($DATA_FILE, "r");
					$aryData = array();
					while ($data = fgetcsv($fp, 1024, ",")) {
						if (strcmp($data[2], $mail) != 0) { // email is the ID
							array_push($aryData, $data);
						}
						else {
							// drop the duplicate (old) record
						}
					}
					fclose($fp);
					$data = array();
					array_push($data, $name, $affi, $mail);
					for ($i=0; $i<$EVENT_NUM; $i++) {
						array_push($data, $type[$i]);
					}
					array_push($data, $date, $code, 0);
					array_push($aryData, $data);
					$fp = fopen($DATA_FILE, "w");
					if ($fp == NULL || !flock($fp, LOCK_EX)) {
						print("<h2>更新エラー</h2><p>データベースが使用中の可能性があります．しばらく待ってからもう一度申し込んでみてください．</p>");
					}
					else { // open and lock OK
						set_file_buffer($fp, 0); // disable file buffer for writing
						foreach($aryData as $data) {
							for ($i=0; $i<$EVENT_NUM+5; $i++) {
								fwrite($fp, $data[$i].",");
							}
							fwrite($fp, $data[$EVENT_NUM+5]."\n");
						}
						flock($fp, LOCK_UN);
						fclose($fp);
					}
					mb_language("Japanese");
					$msg = "$affi\n$name 様\n\n$EVENT_TITLE へお申し込み頂き，ありがとうございます．\n手続きを完了させるには，以下の申込内容をご確認の上，「確認」のURIにアクセスして下さい．\n万が一申込情報が間違っている場合は，同じE-mailアドレスを使って再申込するか，\n以下の「キャンセル」のURIにアクセスして申込を一旦キャンセルし，再入力をお願いします．\n\n";
					for ($i=0; $i<$EVENT_NUM; $i++) {
						$msg = $msg.$EVENT_TIME[$i]." ".$EVENT_NAME[$i].(($type[$i])?" ○ ":" × ")." ";
					}
					$msg = $msg."\n\n確認 $myURI/?action=confirm&code=$code\n\nキャンセル $myURI/?action=cancel&code=$code\n\n";
					$msg = $msg."---\nこのメールは自動送信されたものです．心当たりがない場合，返信せず，\n$myURI \nに記載されている連絡先まで御連絡下さい．\n";
					$msg = mb_convert_encoding($msg, "ISO-2022-JP", "UTF-8,SJIS");
					if (mb_send_mail($mail, "Registration confirmation", $msg, "Return-Path: $ADMIN_EMAIL\r\nFrom: $ADMIN_EMAIL")) {
						print("<h2>参加申込，ステップ２</h2><p><em>$name</em> 様，<em>$mail</em> 宛に確認のメールを送信しました．案内に従って申込の手続きを完了させて下さい．</p>");
					}
					else { // ? When will the else happen??
						print("<h2>送信エラー</h2><p><em>$name</em> 様，<em>$mail</em> 宛に確認のメールを送信しようとしたのですが，エラーとなりました．E-mailアドレスが正しいかをご確認下さい．正しい場合は，ネット障害によるものと考えられるので，上記連絡先までご連絡下さい．</p>");
					}
				}
				else {
					print("<h2>入力エラー</h2><p>最低限氏名とE-mailを教えて下さいね．</p>");
				}
			}
			else {
				print("<h2>不正アクセス</h2><p>[<a href=\".\">トップページ</a>]から申込を行ってください．</p>");
			}
			break;
		///////////////////////////////////////////////////////////////////
		// list
		case "list":
			print<<<EOF
	<h2>参加者一覧</h2>
	<div class="box">
	<table>
		<tr>
			<td><a href="?action=list&amp;sort=0">氏名</a></td>
			<td><a href="?action=list&amp;sort=1">所属</a></td>
EOF;
			for($i=3; $i<$EVENT_NUM+3; $i++) {
				print("<td><a href=\"?action=list&amp;sort=$i\">".$EVENT_NAME[$i-3]."</a></td>");
			}
			print("<td><a href=\"?action=list&amp;sort=".($EVENT_NUM+5)."\">申込状態</a></td></tr>");
			$fp = fopen($DATA_FILE, "r");
			$num = array();
			for ($i=0; $i<$EVENT_NUM; $i++) {
				$num[$i] = 0;
			}
			$aryData = array();
			$n = 0;
			while ($data = fgetcsv($fp, 1024, ",")) {
				$aryData[$n] = $data;
				$n++;
			}
			if (isset($sort)) { // ie., we have query string as action=list&sort=i
				$SORT_FIELD = $sort;
				usort($aryData, data_compare);
			}
			foreach ($aryData as $data) {
				if (!$SHOW_CANCEL && $data[$EVENT_NUM+5] < 0) {
					continue;
				}
				printf("<tr><td>".$data[0]."</td><td>".$data[1]."</td>");
				for($i=3; $i<$EVENT_NUM+3; $i++) {
					if ($data[$i] == "0") {
						print("<td>×</td>");
					}
					else {
						print("<td>○</td>");
						if ($data[$EVENT_NUM+5] >= 0) {
							$num[$i-3]++;
						}
					}
				}
				if ($data[$EVENT_NUM+5] == 0) {
					print("<td>確認待ち</td>");
				}
				else if ($data[$EVENT_NUM+5] > 0) {
					print("<td>確認完了</td>");
				}
				else {
					print("<td>キャンセル</td>");
				}
				print("</tr>");
			}
			fclose($fp);
			print("<tr><td>---</td><td>---</td>");
			for ($i=0; $i<$EVENT_NUM; $i++) {
				print("<td>計 ".$num[$i]." 名</td>");
			}
			print("<td>---</td></tr></table></div>");
			break;
		///////////////////////////////////////////////////////////////////
		// confirmation
		case "confirm":
			if (!isset($code)) {
				print("<h2>確認エラー</h2><p>確認コードがありません．</p>");
				break;
			}
			$fp = fopen($DATA_FILE, "r");
			if ($fp == NULL) {
				print("<h2>更新エラー</h2><p>データベースが使用中のため，しばらく待ってからもう一度アクセスしてみてください．</p>");
				break;
			}
			$index = -1; // index of the data
			$aryData = array();
			$n = 0;
			while ($data = fgetcsv($fp, 1024, ",")) {
				if ($index < 0 && strcmp($data[$EVENT_NUM+4], $code) == 0) {
					$data[$EVENT_NUM+5] = time();
					$index = $n;
				}
				array_push($aryData, $data);
				$n++;
			}
			fclose($fp);
			if ($index < 0) {
				print("<h2>更新エラー</h2><p>確認コードは正しくありません．</p>");
				break;
			}
			$fp = fopen($DATA_FILE, "w");
			if ($fp == NULL || !flock($fp, LOCK_EX)) {
				print("<h2>更新エラー</h2><p>データベースに書き込めません（使用中の可能性があります）．しばらく待ってからもう一度アクセスしてみてください．</p>");
				break;
			}
			set_file_buffer($fp, 0); // disable file buffer for writing
			foreach($aryData as $data) {
				for ($i=0; $i<$EVENT_NUM+5; $i++) {
					fwrite($fp, $data[$i].",");
				}
				fwrite($fp, $data[$EVENT_NUM+5]."\n");
			}
			flock($fp, LOCK_UN);
			fclose($fp);
			$data = $aryData[$index];
			print("<h2>確認完了</h2><p>申込内容：".$data[1]." ".$data[0]." 様");
			for($i=0; $i<$EVENT_NUM; $i++) {
				print(" ".$EVENT_NAME[$i]." ");
				print(($data[$i+3]>0)?"○":"×");
			}
			print("</p>");
			break;
		///////////////////////////////////////////////////////////////////
		// confirmation
		case "cancel":
			if (!isset($code)) {
				print("<h2>キャンセルエラー</h2><p>コードがありません．</p>");
				break;
			}
			$fp = fopen($DATA_FILE, "r");
			if ($fp == NULL) {
				print("<h2>更新エラー</h2><p>データベースが使用中のため，しばらく待ってからもう一度アクセスしてみてください．</p>");
				break;
			}
			$index = -1; // index of the data
			$aryData = array();
			$n = 0;
			while ($data = fgetcsv($fp, 1024, ",")) {
				if ($index < 0 && strcmp($data[$EVENT_NUM+4], $code) == 0) {
					$data[$EVENT_NUM+5] = -time();
					$index = $n;
				}
				array_push($aryData, $data);
				$n++;
			}
			fclose($fp);
			if ($index < 0) {
				print("<h2>更新エラー</h2><p>コードは正しくありません．</p>");
				break;
			}
			$fp = fopen($DATA_FILE, "w");
			if ($fp == NULL || !flock($fp, LOCK_EX)) {
				print("<h2>更新エラー</h2><p>データベースに書き込めません（使用中の可能性があります）．しばらく待ってからもう一度アクセスしてみてください．</p>");
				break;
			}
			set_file_buffer($fp, 0); // disable file buffer for writing
			foreach($aryData as $data) {
				for ($i=0; $i<$EVENT_NUM+5; $i++) {
					fwrite($fp, $data[$i].",");
				}
				fwrite($fp, $data[$EVENT_NUM+5]."\n");
			}
			flock($fp, LOCK_UN);
			fclose($fp);
			$data = $aryData[$index];
			print("<h2>キャンセル完了</h2><p>".$data[1]." ".$data[0]." 様</p>");
			break;
		///////////////////////////////////////////////////////////////////
		// default
		default:
			printf("<h2>不正アクセス！</h2>");
	}
}
print_footer("show address");
?>
